﻿//
//   (c) Copyright Agilent Technologies 2011
//              All rights reserved.
//
//  Customer shall have the personal, non-
// transferable rights to use, copy or modify
// this SAMPLE PROGRAM for Customer's internal
// operations. Customer shall use the SAMPLE
// PROGRAM solely and exclusively for its own
// purpose and shall not license, lease, market
// or distribute the SAMPLE PROGRAM or modification
// or any part thereof.
//
//  Agilent shall not be liable for the quality,
// performance or behavior of the SAMPLE PROGRAM.
// Agilent especially disclaims that the operation
// of the SAMPLE PROGRAM shall be uninterrupted or
// error free. This SAMPLE PROGRAM is provided
// AS IS.
//
//  AGILENT DISCLAIMS THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE.
//
//  Agilent shall not be liable for any infringement
// of any patent, trademark, copyright or other
// proprietary rights by the SAMPLE PROGRAM or
// its use. Agilent does not warrant that the SAMPLE
// PROGRAM is free from infringements or such
// rights of third parties. However, Agilent will not
// knowingly infringe or deliver a software that
// infringes the patent, trademark, copyright or
// other proprietary right of a third party.
//
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace B2900CsSample
{
    public partial class MainWindow : Form
    {
        private SmuControl smu = new SmuControl();

        public MainWindow()
        {
            InitializeComponent();
        }

        private void MainWindow_Load(object sender, EventArgs e)
        {
            // Loading help document from "HelpDocument" resource
            try
            {
                System.Reflection.Assembly asm = this.GetType().Assembly;
                System.Resources.ResourceManager rm = new System.Resources.ResourceManager("B2900CsSample.Properties.Resources", asm);
                string rtfstr = (string)rm.GetObject("HelpDocument");
                richTextBoxHelp.Rtf = rtfstr;
            }
            catch (Exception ex)
            {
                richTextBoxHelp.WordWrap = true;
                richTextBoxHelp.Text = "Cannot find help document." + Environment.NewLine + Environment.NewLine;
                richTextBoxHelp.Text += ex.Message; 
            }
        }

        private void buttonRun_Click(object sender, EventArgs e)
        {
            string address;
            double ch1startv, ch1stopv, ch1compi, ch2forcev, ch2compi;
            int nofstep;

            // Get parameters & Check parameter syntax
            address = comboBoxVisaResource.Text.Trim();
            if ( address == "" )
            {
                MessageBox.Show(this, "VISA Resource String is empty.", "Error");
                return;
            }
            if (!double.TryParse(textBoxCh1StartV.Text, out ch1startv))
            {
                MessageBox.Show(this, "Syntax error in start voltage.", "Error");
                return;
            }
            if (!double.TryParse(textBoxCh1StopV.Text, out ch1stopv))
            {
                MessageBox.Show(this, "Syntax error in stop voltage.", "Error");
                return;
            }
            nofstep = (int)numericUpDownNofStep.Value;
            if (!double.TryParse(textBoxCh1I.Text, out ch1compi))
            {
                MessageBox.Show(this, "Syntax error in channel 1 limit(compliance) current.", "Error");
                return;
            }
            if (!double.TryParse(textBoxCh2V.Text, out ch2forcev))
            {
                MessageBox.Show(this, "Syntax error in channel 2 source voltage.", "Error");
                return;
            }
            if (!double.TryParse(textBoxCh2I.Text, out ch2compi))
            {
                MessageBox.Show(this, "Syntax error in channel 2 limit(compliance) current.", "Error");
                return;
            }

            try
            {
                // Disable buttons
                buttonRun.Enabled = false;
                buttonSave.Enabled = false;

                // Set & Measure
                bool ret = smu.OpenDevice(address);
                textBoxModel.Text = smu.ModelName;
                if (!ret)
                {
                    MessageBox.Show(this, "Failed to open a device." + Environment.NewLine + smu.ErrorMessage, "Error");
                    return;
                }
                if (!smu.SetSweepChannel(1,ch1startv,ch1stopv,nofstep,ch1compi))
                {
                    MessageBox.Show(this, "Failed to set sweep condition." + Environment.NewLine + smu.ErrorMessage, "Error");
                    return;
                }
                if (smu.MaxSmuChannel >= 2)
                {
                    if (!smu.SetConstChannel(2, ch2forcev, ch2compi))
                    {
                        MessageBox.Show(this, "Failed to set channel 2 constant source." + Environment.NewLine + smu.ErrorMessage, "Error");
                        return;
                    }
                }
                if (!smu.Initiate(new int[] {1}))
                {
                    MessageBox.Show(this, "Failed to start measurement." + Environment.NewLine + smu.ErrorMessage, "Error");
                    return;
                }

                // Clear Graph & Table
                dataGridView1.Rows.Clear();
                chart1.Series[0].Points.Clear();

                // Read measured data
                double xmax = double.MinValue;
                double xmin = double.MaxValue;
                double ymax = double.MinValue;
                double ymin = double.MaxValue;
                const int READ_SIZE = 20;
                double[,] data = new double[READ_SIZE, SmuControl.COLUMN_COUNT];
                bool isFirst = true;
                int actrow;
                do
                {
                    if (!smu.GetData(1, isFirst, READ_SIZE, ref data, out actrow))
                    {
                        MessageBox.Show(this, "Failed to read measurement result from the device." + Environment.NewLine + smu.ErrorMessage, "Error");
                        return;
                    }
                    isFirst = false;
                    if (actrow == 0) break;

                    // Update Graph & Table
                    for (int i = 0; i < actrow; i++)
                    {
                        int row = dataGridView1.Rows.Add(data[i, 0], data[i, 1], data[i, 2], data[i, 3], data[i, 4], data[i, 5]);
                        dataGridView1.Rows[row].HeaderCell.Value = (row + 1).ToString();
                        chart1.Series[0].Points.AddXY(data[i, 0], data[i, 1]);
                        if (data[i,0] > xmax) xmax = data[i,0];
                        if (data[i,0] < xmin) xmin = data[i,0];
                        if (data[i,1] > ymax) ymax = data[i,1];
                        if (data[i,1] < ymin) ymin = data[i,1];
                    }
                    scaleAdjust(false, chart1.ChartAreas[0], xmin, xmax, ymin, ymax);
                    Application.DoEvents();
                } while (true);

                scaleAdjust(true, chart1.ChartAreas[0], xmin, xmax, ymin, ymax);
            }
            catch
            {
            }
            finally
            {
                // Close
                smu.CloseDevice();

                // Enable buttons
                buttonRun.Enabled = true;
                buttonSave.Enabled = true;
            }

        }

        private void scaleAdjust(bool isLast, System.Windows.Forms.DataVisualization.Charting.ChartArea chartaria, 
                                 double xDataMin, double xDataMax, double yDataMin, double yDataMax)
        {
            double resolution;
            resolution = Math.Pow(10.0,(int)Math.Floor(Math.Log10(xDataMax - xDataMin)) - 1);
            double new_xAxisMin = Math.Ceiling(xDataMin / resolution) * resolution;
            double new_xAxisMax = Math.Floor(xDataMax / resolution) * resolution;
            resolution = Math.Pow(10.0, (int)Math.Floor(Math.Log10(yDataMax - yDataMin)) - 1);
            double new_yAxisMin = Math.Ceiling(yDataMin / resolution) * resolution;
            double new_yAxisMax = Math.Floor(yDataMax / resolution) * resolution;
            if (isLast)
            {
                chartaria.AxisX.Minimum = new_xAxisMin;
                chartaria.AxisX.Maximum = new_xAxisMax;
                chartaria.AxisY.Minimum = new_yAxisMin;
                chartaria.AxisY.Maximum = new_yAxisMax;
            }
            else
            {
                if (chartaria.AxisX.Minimum > new_xAxisMin) chartaria.AxisX.Minimum = new_xAxisMin;
                if (chartaria.AxisX.Maximum < new_xAxisMax) chartaria.AxisX.Maximum = new_xAxisMax;
                if (chartaria.AxisY.Minimum > new_yAxisMin) chartaria.AxisY.Minimum = new_yAxisMin;
                if (chartaria.AxisY.Maximum < new_yAxisMax) chartaria.AxisY.Maximum = new_yAxisMax;
            }
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            // Modify filename with current timestamp
            string lastfilename = saveFileDialog1.FileName;
            int parenthesis1 = lastfilename.IndexOf('(');
            int parenthesis2 = lastfilename.LastIndexOf(')');
            if (parenthesis1 >= 0 && parenthesis2 >= 0 && parenthesis2 > parenthesis1)
            {
                saveFileDialog1.FileName = lastfilename.Remove(parenthesis1 + 1, parenthesis2 - parenthesis1 - 1)
                                                       .Insert(parenthesis1 + 1,DateTime.Now.ToString("yyyyMMddHHmmss"));
            }

            // Save CVS file
            if (saveFileDialog1.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    // Open file
                    System.IO.StreamWriter tw = new System.IO.StreamWriter(saveFileDialog1.FileName);

                    // Write header
                    int row, col;
                    for (col = 0; col < dataGridView1.Columns.Count; col++)
                    {
                        if (col == 0) tw.Write(string.Format("{0,-8:s}",dataGridView1.Columns[col].HeaderText));
                        else tw.Write("," + string.Format("{0,-8:s}", dataGridView1.Columns[col].HeaderText));
                    }
                    tw.Write(Environment.NewLine);

                    // Write table data
                    for (row = 0; row < dataGridView1.Rows.Count; row++)
                    {
                        for (col = 0; col < dataGridView1.Columns.Count; col++)
                        {
                            if (col == 0) tw.Write(string.Format("{0,-8:s}",dataGridView1.Rows[row].Cells[col].Value.ToString()));
                            else tw.Write("," + string.Format("{0,-8:s}",dataGridView1.Rows[row].Cells[col].Value.ToString()));
                        }
                        tw.Write(Environment.NewLine);
                    }

                    // Close file
                    tw.Close();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(this, "Failed to save CSV file." + Environment.NewLine + ex.Message, "Error");
                }
            }
        }
    }
}
